#!/bin/sh

get_slot_suffix()
{
	slot_suffix_info=$(sed 's/ /\n/g' /proc/cmdline | grep -i hobotboot.slot_suffix | head -n 1)
	slot_suffix=${slot_suffix_info#*=}
}

create_mtd_alias()
{
	disk_info=$(cat "/proc/mtd" | sed '1d' | sed 's/"//g' | sed 's/://g' | awk '{printf("if [ -e /dev/%s ]; then ln -s /dev/%s /dev/block/platform/by-name/%s; fi\n", $1, $1, $4)}')
	sh -c "$disk_info"
	wait
}

link_blk_part()
{
	local blk_dev="$1"
	local partlabel=$(blkid -o value -s PARTLABEL ${blk_dev})
	if [ -n "${partlabel}" ]; then
		ln -sf ${blk_dev} /dev/block/platform/by-name/${partlabel}
	fi
}

create_emmc_alias()
{
	ln -s /dev/mmcblk0 /dev/block/platform/by-name/blk &
	for device in /dev/mmcblk0p*; do
		link_blk_part ${device} &
	done
	wait
}

create_ufs_alias()
{
	local partuuid=$(sed 's/ /\n/g' /proc/cmdline | grep -i partuuid | awk -F "[=/]" '{print $NF}' | head -n 1)
	local device_node=$(realpath /dev/disk/by-partuuid/${partuuid} | sed 's/[0-9]//g')
	echo ${device_node} | grep -q "by-partuuid" && device_node="/dev/sda"
	ln -s ${device_node} /dev/block/platform/by-name/blk &
	for device in ${device_node}*; do
		link_blk_part ${device} &
	done
	wait
}

create_blk_alias()
{
	bootdevice=$(cat /proc/device-tree/soc/boardinfo/bootdevice_name)
	if [ x"$bootdevice" = x"mmc0" ];then
		create_emmc_alias
	else
		create_ufs_alias
	fi
}

create_current_node()
{
	for part_suffix in $(find ${by_name} -name "*_a"); do
	{
		part_name=${part_suffix%_a*}
		ln -sf "${part_name}${slot_suffix}" "${part_name}_cur"
	} &
	done
	wait
}

if [ $# -ge 2 ]; then
	echo "too many arguments."
	exit 1
fi

by_name="/dev/block/platform/by-name/"
slot_suffix=""

mkdir -p ${by_name}

if [ $# -ge 1 ] && [ "$1" = "mtd" ]; then
	create_mtd_alias
else
	create_blk_alias
fi

get_slot_suffix
create_current_node
